@@ -8,10 +8,14 @@ import calendar
 import datetime
 import argparse
 import sys
+import csv
 from seams.severity import Severity
 from traceback import format_tb
 from dotenv import load_dotenv
 
+# The default path and name of the log file
+SEAMS_PIPELINE_LOG_FILE = "pipeline.log" 
+
 load_dotenv()
 class Pipeline(ABC):
     
@@ -45,6 +49,7 @@ class Pipeline(ABC):
 
         self.seams.connect(SECRET, APP_ID, TENANT_ID, AD_AUTH_MODE)
 
+        self.log_file = open(SEAMS_PIPELINE_LOG_FILE, "w")
     
     @abstractmethod
     def run(self, data) -> None:
@@ -108,6 +113,8 @@ class Pipeline(ABC):
             else:
                 return self.update_pipeline_status_done()
 
+            self.log_file.close()
+
 
     def __verify_files(self, data: dict) -> None:
         '''
@@ -313,12 +320,14 @@ class Pipeline(ABC):
             str_log = "{:8} {} - {}".format(log["severity"], date, log["message"])
             if print_out:
                 print(str_log)
+                self.log_file.write(str_log + "\n")
 
             #appends to logs and user_logs if requested
             self.logs.append(log)
             if for_user:
                 self.user_logs.append(log)
 
+
     def log_info(self, *args, for_user=False, print_out=True) -> None:
         """
         Wrapper method for logging info severity messages
@@ -377,6 +386,21 @@ class Pipeline(ABC):
         upload_file_path = os.path.join(tempfile.gettempdir(), new_filename)
 
         return upload_file_path, new_filename
+    
+
+    def __write_logs_to_csv(self, path:str, logs:list) -> None:
+        '''
+        private method to write logs to a CSV file
+
+        :path: string representing the path to a csv file
+        :logs: list of dictionary items that are logs
+        '''
+        with open(path, 'w', newline='') as csvfile:
+            keys = ["severity", "date", "message"]
+            writer = csv.DictWriter(csvfile, keys, delimiter=',', quoting=csv.QUOTE_ALL)
+            writer.writeheader()
+            writer.writerows(logs)
+        csvfile.close()
 
 
     def __build_log_upload_files(self) -> tuple[str, str]:
@@ -388,26 +412,22 @@ class Pipeline(ABC):
         '''
 
         #creates new tempfile for logs
-        logs_name = self.create_new_file("pipeline_logs.json")
+        logs_name = self.create_new_file("pipeline_logs.csv")
         log_temp_file_full_path = os.path.join(tempfile.gettempdir(), logs_name[1])
 
         #opens/writes/closes new temp log file
-        log_file = open(log_temp_file_full_path, "w", encoding="utf-8")
-        log_file.write(str(self.logs))
-        log_file.close()
+        self.__write_logs_to_csv(log_temp_file_full_path, self.logs)
 
         #uploads log file and attaches it as an edge to the pipeline run
         log_file_upload = self.seams.upload_file("logs for {}".format(self.vertex_id), log_temp_file_full_path, file_type="PipelineLogFile")
         self.seams.attach_edges(self.vertex_id, [ log_file_upload["id"] ])
 
         #creates new tempfile for user_logs
-        user_logs_name = self.create_new_file("pipeline_user_logs.json")
+        user_logs_name = self.create_new_file("pipeline_user_logs.csv")
         user_log_temp_file_full_path = os.path.join(tempfile.gettempdir(), user_logs_name[1])
 
         #opens/writes/closes new temp user_log file
-        user_log_file = open(user_log_temp_file_full_path, "w", encoding="utf-8")
-        user_log_file.write(str(self.user_logs))
-        user_log_file.close()
+        self.__write_logs_to_csv(user_log_temp_file_full_path, self.user_logs)
 
         #uploads user_log file and attaches it as an edge to the pipeline run
         user_log_file_upload = self.seams.upload_file("user logs for {}".format(self.vertex_id), user_log_temp_file_full_path, file_type="PipelineLogFile")
