@@ -7,6 +7,7 @@ from seams.exceptions import SeamsException, SeamsAPIException
 import json
 import msal
 import os
+import time
 from requests_toolbelt import MultipartEncoder
 
 class Seams(object):
@@ -25,6 +26,12 @@ class Seams(object):
         self.connected = False
         self.schemas = None
         self.current_schema = None
+        self.retry = self.__check_int(os.getenv("RETRY"))
+        self.wait_time = self.__check_float(os.getenv("WAIT_TIME"))
+        if self.retry == None:
+            self.retry = 5
+        if self.wait_time == None:
+            self.wait_time = .5
 
 
     def connect(self,
@@ -640,12 +647,26 @@ class Seams(object):
 
         request = Request(req_type, api_url, headers=header, data=data, files=files, json=json)
         prepared = session.prepare_request(request)
-        response = session.send(prepared)
+        # Retry the HTTPS request, break loop if it returns good status code
+        for x in range(self.retry):
+            timeout = False
+            try:
+                response = session.send(prepared)
+            except:
+                timeout = True
+                continue
+            # Raise any HTTP response errors (if any)
+            if response.status_code >= 400 or timeout is True:
+                time.sleep(self.wait_time*x)
+                self.connect()
+            else:
+                break
+        
+        if timeout is True:
+            raise TimeoutError("Error: Seams API Timeout   Retry count: ", self.retry, "    Wait time: ", self.wait_time)
 
-        # Raise any HTTP response errors (if any)
         if response.status_code >= 400:
             raise SeamsAPIException("Error {} {}".format(response.status_code, response.text) )
-        #response.raise_for_status()
 
         # Otherwise, return the results
         return response
@@ -656,3 +677,20 @@ class Seams(object):
         for i in range(0, len(data), SIZE):
             yield {k:data[k] for k in islice(it, SIZE)}
 
+
+    def __check_int(self, value):
+        if value is None:
+            return None
+        try:
+            return int(value)
+        except:
+            raise Exception("RETRY env variable should be an integer")
+        
+    def __check_float(self, value):
+        if value is None:
+            return None
+        try:
+            return float(value)
+        except:
+            raise Exception("WAIT_TIME env variable should be a float or an integer")
+
